#execfile ("ga_schedule.py")
import random, time, math
from genetic_optimize import *
from random_optimize import *

def getminutes(t):
    x=time.strptime(t,'%H:%M')
    return x[3]*60+x[4]

def printschedule(r):    
    for d in range(0,len(r),2):        
        name=people[d/2][0]
        origin=people[d/2][1]
        out = flights[(origin,destination)][r[d]]
        ret = flights[(origin,destination)][r[d+1]]
        print '%10s%10s %5s-%5s $%3s %5s-%5s $%3s' % (name,origin,
                                                  out[0],out[1],out[2],
                                                  ret[0],ret[1],ret[2])
def schedule_fitness(sol):
    totalprice=0
    latestarrival=0
    earliestdep=24*60

    for d in range(len(sol)/2):
        # Get the inbound and outbound flights
        origin=people[d][1]
        outbound = flights[(origin,destination)][int(sol[2*d])]
        returnf = flights[(destination,origin)][int(sol[2*d+1])]

        # Total price is the price of all outbound and return flights
        totalprice+=outbound[2]
        totalprice+=returnf[2]

        # Track the latest arrival and earliest departure
        if latestarrival<getminutes(outbound[1]): latestarrival=getminutes(outbound[1])
        if earliestdep>getminutes(returnf[0]): earliestdep=getminutes(returnf[0])

    # Every person must wait at the airport until the latest person arrives.
    # They also must arrive at the same time and wait for their flights.
    totalwait=0
    for d in range(len(sol)/2):
        origin=people[d][1]
        outbound = flights[(origin,destination)][int(sol[2*d])]
        returnf = flights[(destination,origin)][int(sol[2*d+1])]
        totalwait+=latestarrival-getminutes(outbound[1])
        totalwait+=getminutes(returnf[0])-earliestdep
    # Does this solution require an extra day of car rental? That'll be $50!
    if latestarrival < earliestdep: totalprice+=5

    return totalprice+totalwait

def init_solutions_population (population_size, flights_index_ranges):
    
    population=[None]*population_size
    for i in range (population_size):
	random_solution=[]
        
        for j in range (len (flights_index_ranges)):
            min_flight_index=flights_index_ranges[j][0]
            max_flight_index=flights_index_ranges[j][1]
            random_solution.append(random.randint(min_flight_index,max_flight_index))
        population[i]=random_solution

    return population


# Mutation Operation
def mutate_solution (sol, flights_index_range):
    ipos=random.randint(0,len(sol)-1)

    #get random flight from available flights
    min_flight_index=flights_index_range[ipos][0]
    max_flight_index=flights_index_range[ipos][1]
    r=random.randint(min_flight_index,max_flight_index)
    
    sol[ipos]=r
    return sol

   
#Mate operation (crossover)
def solution_crossover(sol1,sol2,params=None):
    ipos=random.randint(1,len(sol1)-2)
    
    return sol1[0:ipos]+sol2[ipos:]


#*******************************************    
#main
people = [('John','BOS'),
          ('Mary','DAL'),
          ('Laura','CAK'),
          ('Abe','MIA'),
          ('Greg','ORD'),
          ('Lee','OMA')]

# LaGuardia airport in New York
destination='LGA'

flights={}
#
for line in file('schedule.txt'):
    origin,dest,depart,arrive,price=line.strip(  ).split(',')
    flights.setdefault((origin,dest),[])

    # Add details to the list of possible flights
    flights[(origin,dest)].append((depart,arrive,int(price)))

flights_index_range=[(0,9)]*(len(people)*2)
solutions_population=init_solutions_population(500,flights_index_range)
random_best=randomoptimize( solutions_population,schedule_fitness )
print "Best random schedule with score %d:" % random_best[0]

printschedule( random_best[1])

#***************************
print "***************** GENETIC ALGORITHM ********************"
solutions_population=init_solutions_population(50,flights_index_range)


best_schedule=genetic_optimize(solutions_population, schedule_fitness, mutate_solution, solution_crossover,
                       0.2,0.2,100,flights_index_range )
printschedule( best_schedule)